<?php

namespace App\Controller;

use App\Entity\Capsule;
use App\Entity\Group;
use App\Entity\User;
use App\Form\CreateCapsuleGroupsFormType;
use App\Form\DeleteGroupFormType;
use App\Form\RemoveGroupFormType;
use App\Form\SelectCapsuleGroupsFormType;
use App\Repository\CapsuleRepository;
use App\Repository\GroupRepository;
use Doctrine\ORM\EntityManagerInterface;
use Exception;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Symfony\Contracts\Translation\TranslatorInterface;
use Symfony\Component\Routing\Annotation\Route;

class CapsuleGroupController extends AbstractController
{
    public function __construct(
        private TranslatorInterface $translator,
        private CapsuleRepository $capsule_repository,
        private EntityManagerInterface $entity_manager,
        private GroupRepository $group_repository
    ) {
    }

    #[Route('/{_locale<%app.supported_locales%>}/my_groups', name:'edit_user_groups')]
    public function editUserGroups(): Response
    {
        $current_user = $this->getUser();

        if (! $current_user instanceof User) {
            return $this->redirectToRoute('app_logout');
        }

        $groups = $current_user->getGroups()->toArray();

        return $this->render('capsule/groups/user_groups.html.twig', [
            'groups' => $groups,
        ]);
    }

    #[Route('/{_locale<%app.supported_locales%>}/my_groups/create', name:'create_group')]
    public function create(Request $request): Response
    {
        $current_user = $this->getUser();

        if (! $current_user instanceof User) {
            return $this->redirectToRoute('app_logout');
        }

        $group = new Group();
        $group->setAuthor($current_user);

        $form = $this->createForm(CreateCapsuleGroupsFormType::class, $group);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $group = $this->createAGroup($form, $group);

            $this->addFlash(
                'success',
                $this->translator->trans(
                    'groups.create.success',
                    [
                        'group_name' => $group->getName()
                    ]
                )
            );

            return $this->redirectToRoute('edit_user_groups');
        }

        return $this->renderForm('capsule/groups/create.html.twig', [
            'createCapsuleGroupsForm' => $form,
        ]);
    }

    /**
     * @throws Exception The group doesn't exist
     */
    #[Route('/{_locale<%app.supported_locales%>}/my_groups/{group_id}/delete', name:'delete_group')]
    public function deleteGroup(int $group_id, Request $request): Response
    {
        $current_user = $this->getUser();
        if (! $current_user instanceof User) {
            return $this->redirectToRoute('app_logout');
        }

        $group = $this->group_repository->findOneBy(['id' => $group_id]);
        if (! $group instanceof Group) {
            throw new Exception('Group not found');
        }

        $form = $this->createForm(DeleteGroupFormType::class);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $capsules = $group->getCapsules()->toArray();
            foreach ($capsules as $capsule) {
                $capsule->removeGroup($group);
                $this->entity_manager->persist($capsule);
            }

            $current_user->removeGroup($group);
            $this->entity_manager->persist($current_user);

            $this->entity_manager->remove($group);
            $this->entity_manager->flush();

            $this->addFlash(
                'success',
                $this->translator->trans(
                    'groups.general.delete.success',
                    [
                        'group_name' => $group->getName()
                    ]
                )
            );

            return $this->redirectToRoute('edit_user_groups');
        }

        return $this->renderForm('capsule/groups/delete.html.twig', [
            'deleteGroupForm' => $form,
            'group_name' => $group->getName(),
        ]);
    }

    /**
     * @throws Exception the capsule doesn't exist
     */
    #[Route('/{_locale<%app.supported_locales%>}/capsule/{capsule_id}/groups/edit', name:'edit_capsule_groups')]
    public function edit(int $capsule_id, Request $request): Response
    {
        $current_user = $this->getUser();

        if (! $current_user instanceof User) {
            return $this->redirectToRoute('app_logout');
        }

        $capsule = $this->capsule_repository->findOneBy(['id' => $capsule_id]);

        if (! $capsule instanceof Capsule) {
            throw new Exception("Capsule does not exist");
        }

        $capsule_groups_by_author = $current_user->getGroupsByCapsule($capsule);
        $author_capsule_groups_not_added = $capsule->getNotAddedGroupsByAuthor($current_user);

        $form = $this->createForm(
            SelectCapsuleGroupsFormType::class,
            ['groups' => $author_capsule_groups_not_added]
        );
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $capsule_groups = $this->addGroups($form, $capsule);

            $groups_names = [];

            foreach ($capsule_groups as $group) {
                $groups_names[] = $group->getName();
            }

            $this->addFlash(
                'success',
                $this->translator->trans(
                    'groups.add.success',
                    [
                        'groups_names' => implode(",", $groups_names),
                        'capsule_name' => $capsule->getName()
                    ]
                )
            );

            return $this->redirectToRoute('edit_capsule_groups', [
                'capsule_id' => $capsule_id,
            ]);
        }

        return $this->renderForm('capsule/groups/capsule_groups.html.twig', [
            'selectCapsuleGroupsForm' => $form,
            'capsule' => $capsule,
            'groups' => $capsule_groups_by_author,
            'groups_not_added' => $author_capsule_groups_not_added
        ]);
    }

    /**
     * @throws Exception the capsule is not found or the group is not found
     */
    #[Route('/{_locale<%app.supported_locales%>}/capsule/{capsule_id}/groups/{group_id}/remove', name:'remove_group')]
    public function remove(
        int $capsule_id,
        int $group_id,
        Request $request
    ): Response {
        $capsule = $this->capsule_repository->findOneBy(['id' => $capsule_id]);
        if (! $capsule instanceof Capsule) {
            throw new Exception('Capsule not found');
        }

        $group = $this->group_repository->findOneBy(['id' => $group_id]);
        if (! $group instanceof Group) {
            throw new Exception('Group not found');
        }

        $current_user = $this->getUser();
        if (! $current_user instanceof User) {
            return $this->redirectToRoute('app_logout');
        }

        $form = $this->createForm(RemoveGroupFormType::class);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $capsule->removeGroup($group);
            $this->entity_manager->persist($capsule);
            $current_user->removeGroup($group);
            $this->entity_manager->persist($current_user);
            $group->removeCapsule($capsule);
            $this->entity_manager->persist($group);
            $this->entity_manager->flush();

            $this->addFlash(
                'success',
                $this->translator->trans(
                    'groups.remove.success',
                    [
                        'group_name' => $group->getName(),
                        'capsule_name' => $capsule->getName()
                    ]
                )
            );

            return $this->redirectToRoute('edit_capsule_groups', [
                'capsule_id' => $capsule->getId()
            ]);
        }

        return $this->renderForm('capsule/groups/remove.html.twig', [
            'removeGroupForm' => $form,
            'group_name' => $group->getName(),
            'capsule_id' => $capsule_id,
        ]);
    }

    /**
     * @return array<Group>
     */
    private function addGroups(FormInterface $form, Capsule $capsule): array
    {
        $groups_data = $form->getData()['name'];
        $groups = [];

        foreach ($groups_data as $group_data) {
            $group_db = $this->group_repository->findOneBy(['id' => $group_data->getId()]);

            if (! $group_db instanceof Group) {
                throw new Exception('Group not found');
            }

            $groups[] = $group_db;
            $capsule->addGroup($group_db);
            $this->entity_manager->persist($capsule);
        }

        $this->entity_manager->flush();

        return $groups;
    }

    private function createAGroup(
        FormInterface $form,
        Group $group
    ): Group {
        $group_name = $form->get('name')->getData();
        $group->setName($group_name);
        $this->entity_manager->persist($group);
        $this->entity_manager->flush();

        return $group;
    }
}
