<?php

namespace App\Controller;

use App\Builder\UserBuilder;
use App\Entity\User;
use App\Event\UserRegistrationEvent;
use App\Form\RegistrationFormType;
use App\Repository\UserRepository;
use App\Security\EmailVerifier;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bridge\Twig\Mime\TemplatedEmail;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Contracts\Translation\TranslatorInterface;
use SymfonyCasts\Bundle\VerifyEmail\Exception\VerifyEmailExceptionInterface;

class RegistrationController extends AbstractController
{
    public function __construct(
        private EmailVerifier $email_verifier,
        private RequestStack $request_stack,
        private EntityManagerInterface $entity_manager,
        private UserRepository $user_repository
    ) {
    }

    #[Route('/register', name:'app_register')]
    public function register(
        Request $request,
        UserPasswordHasherInterface $user_password_hasher,
        EventDispatcherInterface $event_dispatcher
    ): Response {
        if ($this->getUser()) {
            return $this->redirectToRoute('capsule_list');
        }

        $user = new User();
        $form = $this->createForm(RegistrationFormType::class, $user);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $userBuilder = new UserBuilder($user_password_hasher, $user);

            $this->entity_manager->persist($userBuilder->createUser());
            $this->entity_manager->flush();

            $event = new UserRegistrationEvent($user);
            $event_dispatcher->dispatch($event);

            // generate a signed url and email it to the user
            $this->email_verifier->sendEmailConfirmation(
                'app_verify_email',
                $user->getId(),
                $user->getEmail(),
                (new TemplatedEmail())
                    ->to($user->getEmail())
                    ->subject('Please Confirm your Email')
                    ->htmlTemplate('registration/confirmation_email.html.twig')
            );

            $this->request_stack->getSession()->set('userid', $user->getId());
            return $this->redirectToRoute('app_register_mail_sent');
        }

        return $this->renderForm('registration/register.html.twig', [
            'registrationForm' => $form
        ]);
    }

    #[Route('/verify/email', name:'app_verify_email')]
    public function verifyUserEmail(Request $request, TranslatorInterface $translator): Response
    {
        $id = $request->get('id');

        if (null === $id) {
            return $this->redirectToRoute('app_register');
        }

        $user = $this->user_repository->find($id);

        if (null === $user) {
            return $this->redirectToRoute('app_register');
        }

        // validate email confirmation link, sets User::isVerified=true and persists
        try {
            $this->email_verifier->handleEmailConfirmation($request, $user);
        } catch (VerifyEmailExceptionInterface $exception) {
            $this->addFlash('error', $exception->getReason());

            return $this->redirectToRoute('app_register');
        }

        $this->addFlash(
            'success',
            $translator->trans('registration.email_verified_success')
        );

        return $this->redirectToRoute('app_login');
    }

    #[Route('/register_mail_sent', name:'app_register_mail_sent')]
    public function mailSentMessage(): Response
    {
        $user_id = $this->request_stack->getSession()->get('userid');
        $user = $this->user_repository->find($user_id);

        return $this->render(
            'registration/register_mail_sent.html.twig',
            ['user' =>  $user]
        );
    }
}
