<?php

namespace App\Entity;

use App\Entity\User;
use App\Repository\CapsuleRepository;
use App\Retriever\ProjectRetriever;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Doctrine\ORM\Mapping\UniqueConstraint;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\Security\Core\User\UserInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

#[ORM\Table(name:'capsule')]
#[UniqueConstraint(name:'index_capsule_nom', columns:['nom'])]
#[ORM\Entity(repositoryClass:CapsuleRepository::class)]
#[UniqueEntity(fields:['name'], message:'capsule.name.unique')]
class Capsule
{
    #[ORM\Id]
    #[ORM\Column(name:'id', type:'integer')]
    #[ORM\GeneratedValue(strategy:'IDENTITY')]
    private int $id;

    #[ORM\Column(name:'nom', type:'string', length:255, nullable:false)]
    private string $name;

    #[ORM\ManyToOne(targetEntity:User::class)]
    #[ORM\JoinColumn(name:'aut_crea', referencedColumnName:'id', nullable:false)]
    private User $creation_author;

    #[ORM\Column(name:'dt_crea', type:'datetime', nullable:false)]
    private \DateTime $creation_date;

    #[ORM\ManyToOne(targetEntity:'App\Entity\User')]
    #[ORM\JoinColumn(name:'aut_maj', referencedColumnName:'id')]
    private ?User $update_author;

    #[ORM\Column(name:'dt_maj', type:'datetime', nullable:false)]
    private \DateTime $updated_date;

    #[ORM\Column(name:'link', type:'string', length:255, nullable:false)]
    private string $link_path;

    #[ORM\Column(name:'edition_link', type:'string', length:255, nullable:false)]
    private string $edition_link;

    /**
     * @var string the capsule edit password access
     */
    #[ORM\Column(name:'password', type:'string', length:50, nullable:false)]
    private string $password = '';

    /**
     * @var Collection<User>
     */
    #[ORM\ManyToMany(targetEntity:User::class, inversedBy:'capsules')]
    #[ORM\JoinTable(name:'editeur_capsule')]
    #[ORM\JoinColumn(name:'capsule_id', referencedColumnName:'id')]
    #[ORM\InverseJoinColumn(name:'user_id', referencedColumnName:'id')]
    private Collection $editors;

    /**
     * @var Collection<Group>
     */
    #[ORM\ManyToMany(targetEntity:Group::class, mappedBy:'capsules')]
    private Collection $groups;

    public function __construct()
    {
        $this->editors = new ArrayCollection();
        $this->groups = new ArrayCollection();
    }

    public function getId(): int
    {
        return $this->id;
    }

    public function setId(int $id): Capsule
    {
        $this->id = $id;
        return $this;
    }

    public function getName(): string
    {
        return $this->name;
    }

    public function setName(string $name): Capsule
    {
        $this->name = $name;
        return $this;
    }

    public function getCreationAuthor(): User
    {
        return $this->creation_author;
    }

    public function setCreationAuthor(User $creation_author): Capsule
    {
        $this->creation_author = $creation_author;
        return $this;
    }

    public function getCreationDate(): \DateTime
    {
        return $this->creation_date;
    }

    public function setCreationDate(\DateTime $creation_date): Capsule
    {
        $this->creation_date = $creation_date;
        return $this;
    }

    public function getLinkPath(): string
    {
        return $this->link_path;
    }

    public function setLinkPath(string $link_path): Capsule
    {
        $this->link_path = $link_path;
        return $this;
    }

    public function getEditionLink(): string
    {
        return $this->edition_link;
    }

    public function setEditionLink(string $edition_link): Capsule
    {
        $this->edition_link = $edition_link;
        return $this;
    }

    public function getUpdateAuthor(): ?User
    {
        return $this->update_author;
    }

    public function setUpdateAuthor(?User $update_author): void
    {
        $this->update_author = $update_author;
    }

    public function getUpdatedDate(): \DateTime
    {
        return $this->updated_date;
    }

    public function setUpdatedDate(\DateTime $update_date): void
    {
        $this->updated_date = $update_date;
    }

    public function getPassword(): string
    {
        return $this->password;
    }

    public function setPassword(string $password): void
    {
        $this->password = $password;
    }

    public function addEditor(User $editor): void
    {
        $editor->addCapsule($this);
        $this->editors[] = $editor;
    }

    public function removeEditor(User $editor): Capsule
    {
        $editor->removeCapsule($this);
        $this->editors->removeElement($editor);
        return $this;
    }

    /**
     * @return Collection<User>
     */
    public function getEditors(): Collection
    {
        return $this->editors;
    }

    public function getVideoPreviewImageLink(HttpClientInterface $http_client): ?string
    {
        $file_system = new Filesystem();

        if (! $file_system->exists('../legacy/' . $this->getLinkPath())) {
            return null;
        }

        try {
            $project_retriever = new ProjectRetriever($this);
            $video_url = $project_retriever->getVideoUrl();
            $video_id = $project_retriever->getVideoId($video_url);

            if (str_contains($video_url, 'yout')) {
                $url = 'https://img.youtube.com/vi/%s/%s.jpg';
                $high_resolution = sprintf($url, $video_id, 'maxresdefault');
                $medium_resolution = sprintf($url, $video_id, 'mqdefault');
                try {
                    $http_client->request('GET', $high_resolution);
                    return $high_resolution;
                } catch (\Exception $e) {
                    return $medium_resolution;
                }
            }

            return 'https://vumbnail.com/' . $video_id . '.jpg';
        } catch (\Exception $e) {
            return null;
        }
    }

    /**
     * @return Collection<Group>>
     */
    public function getGroups(): Collection
    {
        return $this->groups;
    }

    public function addGroup(Group $group): self
    {
        if (!$this->groups->contains($group)) {
            $this->groups[] = $group;
            $group->addCapsule($this);
        }

        return $this;
    }

    public function removeGroup(Group $group): self
    {
        if ($this->groups->removeElement($group)) {
            $group->removeCapsule($this);
        }

        return $this;
    }

    /**
     * @return array<Group>
     */
    public function getNotAddedGroupsByAuthor(User $author): array
    {
        return array_udiff(
            $author->getGroups()->toArray(),
            $this->getGroups()->toArray(),
            function ($obj1, $obj2) {
                return $obj1->getId() - $obj2->getId();
            }
        );
    }
}
