<?php

namespace App\Tests\functional;

use App\Entity\Capsule;
use App\Entity\User;
use App\Tests\TestFileHelper;
use Symfony\Bundle\FrameworkBundle\Test\WebTestCase;
use Symfony\Component\Filesystem\Filesystem;

class CapsuleControllerTest extends WebTestCase
{
    private Capsule $created_capsule;
    private User $verified_user;
    private const TEST_DIR_PATH = __DIR__ . '/../../legacy/';
    private const CAPSULE_NAME = 'CapsuleToTest';

    protected function setUp(): void
    {
        TestFileHelper::createLegacyDirectoryIfNecessary($this);
        chdir(self::TEST_DIR_PATH);

        self::ensureKernelShutdown();
        $client = static::createClient();

        $object_manager = $client->getContainer()
            ->get('doctrine')
            ->getManager();

        $verified_user = $object_manager
            ->getRepository(User::class)
            ->findOneBy(['email' => 'defaultUser@localhost.com']);

        if (! $verified_user instanceof User) {
            throw new \Exception("User does not exist.");
        }

        $this->verified_user = $verified_user;
        $client->loginUser($this->verified_user);

        $crawler = $client->request('GET', '/en/create');
        $this->assertResponseIsSuccessful();

        $client->enableProfiler();

        $submit_button = $crawler->selectButton('Create a capsule');
        $form = $submit_button->form();

        $video_url = "https://www.youtube.com/watch?v=dQw4w9WgXcQ&ab_channel=RickAstley";
        $form['create_capsule_form[name]'] = self::CAPSULE_NAME;
        $form['create_capsule_form[video_url]'] = $video_url;

        $client->submit($form);

        $this->assertResponseRedirects(
            '/en/my_capsules',
            302,
            'Once the capsule is created, the user should be redirected to its capsules lists'
        );

        $client->followRedirect();
        $this->assertResponseIsSuccessful('/en/my_capsules');

        $capsule_repository = $object_manager->getRepository(Capsule::class);
        $capsule_in_db = $capsule_repository->findOneBy(['name' => self::CAPSULE_NAME]);

        if (! $capsule_in_db instanceof Capsule) {
            throw new \Exception("Capsule does not exist.");
        }
        $this->created_capsule = $capsule_in_db;
        self::ensureKernelShutdown();
    }

    protected function tearDown(): void
    {
        $file_system = new Filesystem();

        $file_system->remove(self::TEST_DIR_PATH . $this->created_capsule->getLinkPath());

        $object_manager = $this->getContainer()
            ->get('doctrine')
            ->getManager();

        $last_capsule = $object_manager->merge($this->created_capsule);
        $object_manager->remove($last_capsule);
        $object_manager->flush();
        parent::tearDown();
    }

    public function testANonAuthenticatedUserShouldNotBeAbleToAccessTheCapsuleListPage(): void
    {
        $client = static::createClient();
        $client->request('GET', '/en/my_capsules');

        $this->assertResponseRedirects("/login", 302);
    }

    public function testNonLoggedUserShouldAccessToCapsulePreview(): void
    {
        $client = static::createClient();

        $client->request('GET', '/en/capsule/preview/' . $this->created_capsule->getLinkPath());
        $this->assertResponseIsSuccessful('The preview should be allowed for none authenticated user');
    }

    public function testLoggedUserShouldAccessToCapsulePreview(): void
    {
        $client = static::createClient();
        $client->loginUser($this->verified_user);

        $client->request('GET', '/en/capsule/preview/' . $this->created_capsule->getLinkPath());
        $this->assertResponseIsSuccessful('The preview should be allowed for none authenticated user');
    }

    public function testLoggedUserShouldAccessToAnUnExistingCapsulePreview(): void
    {
        $client = static::createClient();
        $client->loginUser($this->verified_user);

        $client->request('GET', '/en/capsule/preview/' . substr($this->created_capsule->getLinkPath(), -6));
        $this->assertResponseIsSuccessful('The preview should be allowed for non authenticated user');
    }

    public function testNonLoggedUserShouldNotAccessToCapsuleEdition(): void
    {
        $client = static::createClient();

        $client->request('GET', '/en/capsule/edit/' . $this->created_capsule->getLinkPath());
        $this->assertResponseRedirects(
            '/login',
            302,
            'An unauthenticated user should no access to capsule edition and should be redirected to the login page'
        );
    }

    public function testLoggedUserShouldAccessToItsCapsuleEdition(): void
    {
        $client = static::createClient();
        $client->loginUser($this->verified_user);

        $client->request('GET', '/en/capsule/edit/' . $this->created_capsule->getLinkPath());
        $this->assertResponseIsSuccessful(
            'An authenticated user should be able to access to the edition of its capsules'
        );
    }
}
